#pragma once

#define SQLITE_PREPARE(db, stmt, SQL, goto_label)                       \
    if (sqlite3_prepare_v2(db, SQL, -1, &(stmt), 0) != SQLITE_OK) {     \
        fprintf(stderr, " ! %s: Failed to prepare statement: %s\n",     \
                __func__, sqlite3_errmsg(db));                          \
        goto goto_label;                                                \
    }

#define SQLITE_EXEC(db, SQL, goto_label)                                \
    if (sqlite3_exec(db, SQL, NULL, NULL, NULL) != SQLITE_OK) {         \
        fprintf(stderr, " ! %s: Failed to exec statement: %s\n",        \
                __func__, sqlite3_errmsg(db));                          \
        goto goto_label;                                                \
    }

#define SQLITE_BIND_TEXT(db, stmt, pos, text, error)                    \
    if (sqlite3_bind_text(stmt, pos, text, -1, 0) != SQLITE_OK) {       \
        fprintf(stderr, " ! %s: Failed to bind text %s at position"     \
                "%d: %s\n", __func__, text, pos, sqlite3_errmsg(db));   \
        goto error;                                                     \
    }

#define SQLITE_BIND_INT(db, stmt, pos, integer, error)                  \
    if (sqlite3_bind_int(stmt, pos, integer) != SQLITE_OK) {            \
        fprintf(stderr, " ! %s: Failed to bind int %d at position %d:"  \
                "%s\n", __func__, integer, pos, sqlite3_errmsg(db));    \
        goto error;                                                     \
    }

#define SQLITE_STEP(db, stmt, code, error)                              \
    if (sqlite3_step(stmt) != code) {                                   \
        fprintf(stderr, " ! %s: Failed to step and get a row: %s\n",    \
                __func__, sqlite3_errmsg(db));                          \
        goto error;                                                     \
    }

#define SQLITE_STEP_ROW(db, stmt, error)    SQLITE_STEP(db, stmt, SQLITE_ROW, error)
#define SQLITE_STEP_DONE(db, stmt, error)   SQLITE_STEP(db, stmt, SQLITE_DONE, error)
#define SQLITE_STEP_OK(db, stmt, error)     SQLITE_STEP(db, stmt, SQLITE_OK, error)

#define SQLITE_DO_ROLLBACK(db)                                          \
    sqlite3_exec(db, "ROLLBACK TRANSACTION;\n", NULL, NULL, NULL);

#ifndef MAX
#define MAX(a, b) ((a) < (b) ? (b) : (a))
#endif /* MAX */

#ifndef MIN
#define MIN(a, b) ((a) > (b) ? (b) : (a))
#endif /* MIN */
