#pragma once

#include <lektor/defines.h>
#include <stdbool.h>
#include <sqlite3.h>

/* The real window struct, an opaque type */
typedef void *lkt_win_element_t;

/* The window with the selected correct functions, this is done because we may
 * need an SDL2, a X11 or a QT window, and may want to select the one to use at
 * run time. */
struct lkt_win {
    lkt_win_element_t window;           /* the opaque window type that depend on the module */
    void *handle;                       /* the libdl handle                                 */

    /* Create and free the window */
    bool (*new)(struct lkt_win *win);   /* Create a window or only the mpv context          */
    void (*close)(struct lkt_win *win); /* Close the mpv context, not the window.           */
    void (*free)(struct lkt_win *win);  /* Entirelly liberate all the resources             */

    /* Playback control */
    bool (*toggle_pause)(struct lkt_win *win, bool *new_paused);
    bool (*load_file)(struct lkt_win *win, const char *filepath);
    bool (*set_volume)(struct lkt_win *win, int vol);

    /* Get playback properties */
    bool (*is_paused)(struct lkt_win *win, bool *ret);
    bool (*get_duration)(struct lkt_win *win, int *dur_sec);
    bool (*get_elapsed)(struct lkt_win *win, int *elapsed_sec);

    /* Handle loop event */
    bool (*handle_events)(struct lkt_win *win, sqlite3 *db, enum mpd_idle_flag *mpd_idle_events);
};

enum lkt_win_type {
    lkt_win_type_none = 0,
    lkt_win_type_x11 = 1,
};
