#include <lektor/uri.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>

bool
lkt_uri_from(struct lkt_uri_t *ret, char *const str)
{
    char *val, *endptr;
    size_t len;
    long id;

    if (ret == NULL || str == NULL)
        return false;

    len = strlen(str);

    // The minimal uri is id://1 which is a 6 characters long
    if (len <= 5 * sizeof(char)) {
        fprintf(stderr, " ! lkt_uri_from: candidate for uri is to short to be an uri: %s\n", str);
        return false;
    }

    if (!strncasecmp(str, "fs", 2 * sizeof(char))) {
        ret->type = uri_fs;
        val = str + 2;
    } else if (!strncasecmp(str, "id", 2 * sizeof(char))) {
        ret->type = uri_id;
        val = str + 2;
    } else if (!strncasecmp(str, "plt", 3 * sizeof(char))) {
        ret->type = uri_playlist;
        val = str + 3;
    } else if (!strncasecmp(str, "type", 4 * sizeof(char))) {
        ret->type = uri_type;
        val = str + 4;
    } else if (!strncasecmp(str, "author", 6 * sizeof(char))) {
        ret->type = uri_author;
        val = str + 6;
    } else if (!strncasecmp(str, "category", 8 * sizeof(char))) {
        ret->type = uri_category;
        val = str + 8;
    } else if (!strncasecmp(str, "lang", 4 * sizeof(char))) {
        ret->type = uri_language;
        val = str + 4;
    } else if (!strncasecmp(str, "query", 5 * sizeof(char))) {
        ret->type = uri_query;
        val = str + 5;
    } else {
        fprintf(stderr, " ! lkt_uri_from: unknown uri type for %s\n", str);
        return false;
    }

    // No place for the '://' separator
    if (len - (val - str) < 3) {
        fprintf(stderr, " ! lkt_uri_from: no separator '://' for uri %s\n", str);
        return false;
    }

    val += 3;

    if (ret->type == uri_id) {
        id = strtol(val, &endptr, 10);

        if ((errno == ERANGE && (id == LONG_MAX || id == LONG_MIN))
            || (errno != 0 && id == 0) || endptr == str)
            return false;

        ret->value = calloc(1, sizeof(int));

        if (ret->value == NULL)
            return false;

        *(int *) ret->value = id;
        ret->_allocated = true;
    }

    else {
        ret->_allocated = false;
        ret->value = val;
    }

    return true;
}

void
lkt_uri_free(struct lkt_uri_t *ret)
{
    if (NULL == ret)
        return;

    if (ret->_allocated)
        free(ret->value);
}
